"""
Functions for dealing with markup text

Taken from scrapy
"""

import re
from htmlentitydefs import name2codepoint

def str_to_unicode(text, encoding=None):
    """Return the unicode representation of text in the given encoding. Unlike
    .encode(encoding) this function can be applied directly to a unicode
    object without the risk of double-decoding problems (which can happen if
    you don't use the default 'ascii' encoding)
    """

    if encoding is None:
        encoding = 'utf-8'
    if isinstance(text, str):
        return text.decode(encoding)
    elif isinstance(text, unicode):
        return text
    else:
        raise TypeError('str_to_unicode must receive a str or unicode object, got %s' % type(text).__name__)

def unicode_to_str(text, encoding=None):
    """Return the str representation of text in the given encoding. Unlike
    .encode(encoding) this function can be applied directly to a str
    object without the risk of double-decoding problems (which can happen if
    you don't use the default 'ascii' encoding)
    """

    if encoding is None:
        encoding = 'utf-8'
    if isinstance(text, unicode):
        return text.encode(encoding)
    elif isinstance(text, str):
        return text
    else:
        raise TypeError('unicode_to_str must receive a unicode or str object, got %s' % type(text).__name__)


_ent_re = re.compile(r'&(#?(x?))([^&;\s]+);')
_tag_re = re.compile(r'<[a-zA-Z\/!].*?>', re.DOTALL)

def remove_entities(text, keep=(), remove_illegal=True, encoding='utf-8'):
    """Remove entities from the given text.

    'text' can be a unicode string or a regular string encoded in the given
    `encoding` (which defaults to 'utf-8').

    If 'keep' is passed (with a list of entity names) those entities will
    be kept (they won't be removed).

    It supports both numeric (&#nnnn; and &#hhhh;) and named (&nbsp; &gt;)
    entities.

    If remove_illegal is True, entities that can't be converted are removed.
    If remove_illegal is False, entities that can't be converted are kept "as
    is". For more information see the tests.

    Always returns a unicode string (with the entities removed).
    """

    def convert_entity(m):
        entity_body = m.group(3)
        if m.group(1):
            try:
                if m.group(2):
                    number = int(entity_body, 16)
                else:
                    number = int(entity_body, 10)
                # Numeric character references in the 80-9F range are typically
                # interpreted by browsers as representing the characters mapped
                # to bytes 80-9F in the Windows-1252 encoding. For more info
                # see: http://en.wikipedia.org/wiki/Character_encodings_in_HTML
                if 0x80 <= number <= 0x9f:
                    return chr(number).decode('cp1252')
            except ValueError:
                number = None
        else:
            if entity_body in keep:
                return m.group(0)
            else:
                number = name2codepoint.get(entity_body)
        if number is not None:
            try:
                return unichr(number)
            except ValueError:
                pass

        return u'' if remove_illegal else m.group(0)

    return _ent_re.sub(convert_entity, str_to_unicode(text, encoding))

def has_entities(text, encoding=None):
    return bool(_ent_re.search(str_to_unicode(text, encoding)))

def replace_tags(text, token='', encoding=None):
    """Replace all markup tags found in the given text by the given token. By
    default token is a null string so it just remove all tags.

    'text' can be a unicode string or a regular string encoded as 'utf-8'

    Always returns a unicode string.
    """
    return _tag_re.sub(token, str_to_unicode(text, encoding))


def remove_comments(text, encoding=None):
    """ Remove HTML Comments. """
    return re.sub('<!--.*?-->', u'', str_to_unicode(text, encoding), re.DOTALL)

def remove_tags(text, which_ones=(), keep=(), encoding=None):
    """ Remove HTML Tags only.

        which_ones and keep are both tuples, there are four cases:

        which_ones, keep (1 - not empty, 0 - empty)
        1, 0 - remove all tags in which_ones
        0, 1 - remove all tags except the ones in keep
        0, 0 - remove all tags
        1, 1 - not allowd
    """

    assert not (which_ones and keep), 'which_ones and keep can not be given at the same time'

    def will_remove(tag):
        if which_ones:
            return tag in which_ones
        else:
            return tag not in keep

    def remove_tag(m):
        tag = m.group(1)
        return u'' if will_remove(tag) else m.group(0)

    regex = '</?([^ >/]+).*?>'
    retags = re.compile(regex, re.DOTALL | re.IGNORECASE)

    return retags.sub(remove_tag, str_to_unicode(text, encoding))

def remove_tags_with_content(text, which_ones=(), encoding=None):
    """ Remove tags and its content.

        which_ones -- is a tuple of which tags with its content we want to remove.
                      if is empty do nothing.
    """
    text = str_to_unicode(text, encoding)
    if which_ones:
        tags = '|'.join([r'<%s.*?</%s>|<%s\s*/>' % (tag, tag, tag) for tag in which_ones])
        retags = re.compile(tags, re.DOTALL | re.IGNORECASE)
        text = retags.sub(u'', text)
    return text


def replace_escape_chars(text, which_ones=('\n', '\t', '\r'), replace_by=u'', \
        encoding=None):
    """ Remove escape chars. Default : \\n, \\t, \\r

        which_ones -- is a tuple of which escape chars we want to remove.
                      By default removes \n, \t, \r.

        replace_by -- text to replace the escape chars for.
                      It defaults to '', so the escape chars are removed.
    """
    for ec in which_ones:
        text = text.replace(ec, str_to_unicode(replace_by, encoding))
    return str_to_unicode(text, encoding)

def unquote_markup(text, keep=(), remove_illegal=True, encoding=None):
    """
    This function receives markup as a text (always a unicode string or a utf-8 encoded string) and does the following:
     - removes entities (except the ones in 'keep') from any part of it that it's not inside a CDATA
     - searches for CDATAs and extracts their text (if any) without modifying it.
     - removes the found CDATAs
    """
    _cdata_re = re.compile(r'((?P<cdata_s><!\[CDATA\[)(?P<cdata_d>.*?)(?P<cdata_e>\]\]>))', re.DOTALL)

    def _get_fragments(txt, pattern):
        offset = 0
        for match in pattern.finditer(txt):
            match_s, match_e = match.span(1)
            yield txt[offset:match_s]
            yield match
            offset = match_e
        yield txt[offset:]

    text = str_to_unicode(text, encoding)
    ret_text = u''
    for fragment in _get_fragments(text, _cdata_re):
        if isinstance(fragment, basestring):
            # it's not a CDATA (so we try to remove its entities)
            ret_text += remove_entities(fragment, keep=keep, remove_illegal=remove_illegal)
        else:
            # it's a CDATA (so we just extract its content)
            ret_text += fragment.group('cdata_d')
    return ret_text



from itertools import ifilter, imap
def linebyline(x):
    return set(imap(str.strip, ifilter(lambda x: x and not x.startswith('#'), x.split('\n'))))


conjunctions = """
for
and
nor
but
or
yet
so
after
because
although
if
before
since
though
unless
when
now that
even though
only if
while
as
whereas
whether or not
since
in order that
while
even if
until
so
in case
in case that
"""

prepositions = """
aboard
about
above
absent
across
after
against
along
alongside
amid
amidst
among
amongst
around
as
astride
at
atop
barring
before
behind
below
beneath
beside
besides
between
beyond
but
by
despite
down
during
except
failing
following
for
from
in
inside
into
like
mid
minus
near
next
notwithstanding
of
off
on
onto
opposite
outside
over
past
plus
regarding
round
save
since
than
through
throughout
till
times
to
toward
towards
under
underneath
unlike
until
up
upon
via
with
within
without
according to
ahead of
as to
aside from
because of
close to
due to
far from
in to
inside of
instead of
near to
next to
on to
out of
outside of
owing to
prior to
subsequent to
as far as
as well as
by means of
in accordance with
in addition to
in front of
in place of
in spite of
on account of
on behalf of
on top of
with regard to
in case of
"""

numbers = """
zero
one
two
three
four
five
six
seven
eight
nine
ten
eleven
twelve
dozen
thirteen
fourteen
fifteen
sixteen
seventeen
eighteen
nineteen
twenty
thirty
fourty
fifty
sixty
seventy
eighty
ninety
hundred
thousand
million
billion
trillion
"""

pronouns = """
he
she
it
him
her
his
hers
its
they
them
their
theirs
I
me
mine
we
us
our
ours
you
your
yours
one
one's
ones
anyone
anybody
anything
someone
somebody
something
everyone
everybody
everything
nothing
nobody
any
each
either
neither
all
most
some
several
none
both
few
many
himself
herself
itself
themselves
themself
myself
ourselves
ourself
oneself
this
that
these
those
who
whom
which
whose
whoever
whomever
whatever
whichever
what
"""

honorifics = """
A.
Adj.
Adm.
Adv.
Asst.
B.
Bart.
Bldg.
Brig.
Bros.
C.
Capt.
Cmdr.
Col.
Comdr.
Con.
Cpl.
D.
DR.
Dr.
E.
Ens.
F.
G.
Gen.
Gov.
H.
Hon.
Hosp.
I.
Insp.
J.
K.
L.
Lt.
M.
M.
MM.
MR.
MRS.
MS.
Maj.
Messrs.
Mlle.
Mme.
Mr.
Mrs.
Ms.
Msgr.
N.
O.
Op.
Ord.
P.
Pfc.
Ph.
Prof.
Pvt.
Q.
R.
Rep.
Reps.
Res.
Rev.
Rt.
S.
Sen.
Sens.
Sfc.
Sgt.
Sr.
St.
Supt.
Surg.
T.
U.
V.
W.
X.
Y.
Z.
v.
vs.
"""

scientific_units = """
kg
kilogram
key
hectogram
dekagram
gram
g
decigram
centigram
milligram
mg
microgram
tonne
metric ton
megagram
kilotonne
gigagram
teragram
carat
ct
amu
atomic mass unit
pound
lb
lbm
ounce
oz
lid
ton
kiloton
slug
stone
grain
m
meter
metre
decimeter
cm
centimeter
mm
millimeter
micrometer
micron
nanometer
nm
dekameter
hectometer
km
kilometer
megameter
angstrom
fermi
inch
in
inches
mil
microinch
microinches
foot
ft
feet
yard
yd
mile
mi
nautical mile
nmi
league
chain
fathom
#rod
furlong
hand
cubit
point
pica
caliber
football field
marathon
au
astronomical unit
light year
light minute
light second
parsec
kiloparsec
megaparsec
screw size
AWG
American Wire Gauge
standard gauge
zinc gauge
ring size
shoe size mens
shoe size womens
s
sec
second
ms
millisecond
microsecond
ns
nanosecond
minute
min
hour
hr
bell
watch
watches
day
week
wk
fortnight
month
year
yr
calendar year
decade
century
centuries
millennium
millennia
man hour
man week
man month
man year
K
Kelvin
deg K
degree Kelvin
C
Celsius
deg C
degree Celsius
R
Rankine
deg R
F
Fahrenheit
deg F
degree Fahrenheit
C deg
Celsius degree
F deg
Fahrenheit degree
A
ampere
amp
milliampere
milliamp
mA
microampere
kiloampere
kA
coulomb
amp hour
mAh
milliamp hour
volt
V
millivolt
mV
kilovolt
kV
ohm
milliohm
microhm
kilohm
siemens
farad
millifarad
microfarad
nanofarad
picofarad
weber
Wb
henry
H
millihenry
mH
microhenry
tesla
T
mol
mole
gram mole
kilomole
kmol
pound mole
lbmol
avogadro
cd
candela
lumen
lm
lux
footcandle
metercandle
lambert
millilambert
footlambert
radian
rad
circle
turn
revolution
rev
degree
deg
arc min
arc minute
min arc
minute arc
arc sec
arc second
sec arc
second arc
quadrant
right angle
gradian
sr
steradian
sphere
hemisphere
bit
kilobit
byte
B
kilobyte
kB
megabyte
MB
gigabyte
GB
terabyte
TB
petabyte
PB
kibibyte
KiB
mebibyte
MiB
gibibyte
GiB
tebibyte
TiB
pebibyte
PiB
bps
kbps
unit
pi
pair
hat trick
dozen
doz
bakers dozen
score
gross
great gross
ream
percent
%
mill
APR
proof
ppm
parts per million
ppb
parts per billion
ppt
parts per trillion
karat
carat gold
newton
N
dekanewton
kilonewton
kN
meganewton
millinewton
dyne
kg force
kgf
kilogram force
gram force
pound force
lbf
ton force
ounce force
ozf
barn
are
decare
dekare
hectare
acre
section
township
homestead
circular inch
circular mil
cc
cubic centimeter
liter
l
litre
deciliter
centiliter
milliliter
ml
dekaliter
hectoliter
kiloliter
kl
megaliter
gallon
gal
quart
qt
pint
pt
fluid ounce
fl oz
ounce fluid
imperial gallon
imp gal
imperial quart
imp qt
imperial pint
imp pt
imperial fluid ounce
imp fl oz
cup
tablespoon
tbsp
teaspoon
tsp
barrel
bbl
shot
fifth
wine bottle
magnum
keg
bushel
peck
cord
board foot
board feet
knot
kt
light speed
mph
kph
mach
rpm
rps
gph
gpm
cfs
cfm
lpm
sccm
sccs
slpm
slph
scfh
scfm
Pa
pascal
kPa
kilopascal
MPa
megapascal
atm
atmosphere
bar
mbar
millibar
microbar
decibar
kilobar
mm Hg
millimeter of Hg
torr
in Hg
inch of Hg
m water
m H2O
meter of water
in water
in H2O
inch of water
ft water
ft H2O
feet of water
foot of head
ft hd
psi
pound per sq inch
ksi
density water
density sea water
density Hg
density air
density steel
density aluminum
density zinc
density brass
density copper
density iron
density nickel
density tin
density titanium
density silver
density nylon
density polycarbonate
joule
J
kilojoule
kJ
megajoule
gigajoule
millijoule
mJ
calorie
cal
kilocalorie
kcal
calorie food
Btu
British thermal unit
erg
electronvolt
eV
kWh
kilowatt hour
ton TNT
watt
W
kilowatt
kW
megawatt
MW
gigawatt
GW
milliwatt
horsepower
hp
metric horsepower
hertz
Hz
millihertz
kilohertz
kHz
megahertz
MHz
gigahertz
GHz
becquerel
Bq
curie
millicurie
roentgen
gray
Gy
rad. abs. dose
sievert
millisievert
Sv
rem
millirem
poise
P
centipoise
cP
stokes
St
centistokes
cSt
gravity
gravity constant
gas constant
mpg
m.p.g.
m.p.s.
bps
b.p.s.
kB/
kb/s
"""

numbers          = linebyline(numbers)
pronouns         = linebyline(pronouns)
prepositions     = linebyline(prepositions)
conjunctions     = linebyline(conjunctions)
honorifics       = linebyline(honorifics)
scientific_units = linebyline(scientific_units)




honorifics = frozenset(map(str.strip, """
Doctor Dr Dr.
Mr Mr.
Miss Ms Ms.
Mrs Mrs.
Captain Capt. Capt
Professor Prof Prof.
Reverend Rev Rev.
Senator Sen Sen.
""".split()))

honorifics_all = """
#A.
Adj.
Adm.
Adv.
Asst.
#B.
Bart.
Bldg.
Brig.
Bros.
C.
Capt.
Cmdr.
Col.
Comdr.
Con.
Cpl.
D.
DR.
Dr.
E.
Ens.
F.
G.
Gen.
Gov.
H.
Hon.
Hosp.
I.
Insp.
J.
K.
L.
Lt.
M.
M.
MM.
MR.
MRS.
MS.
Maj.
Messrs.
Mlle.
Mme.
Mr.
Mrs.
Ms.
Msgr.
N.
O.
Op.
Ord.
P.
Pfc.
Ph.
Prof.
Pvt.
Q.
R.
Rep.
Reps.
Res.
Rev.
Rt.
S.
Sen.
Sens.
Sfc.
Sgt.
Sr.
St.
Supt.
Surg.
T.
U.
V.
W.
X.
Y.
Z.
v.
vs.
"""



"""
When an exception is encountered, save context information about the exception
to a file, so that we can later invoke an editor on the last exception, bringing
it automatically at the correct file:line location.

Install this feature in your sitecustomize.py file by importing this module.
"""

import re, os, sys, traceback, subprocess
from os.path import join, isabs, realpath

__all__ = ('invoke_editor',)

import atexit

def enable(editor=False):
    atexit.register(atexit_handler)
    if editor:
        atexit.register(invoke_editor)


def get_filename():
    return join(os.environ.get('HOME', '/tmp'), '.python_last_exception')


def atexit_handler():
    if not hasattr(sys, 'last_traceback'):
        return
    else:
        cwd = os.environ.get('PWD', '')
        fn = get_filename()
        f = open(fn, 'w')
        f.write(cwd + '\n')
        traceback.print_tb(sys.last_traceback, file=f)
        f.close()


def invoke_editor():

    try:
        f = open(get_filename())
    except IOError:
        print >> sys.stderr, "error: can't find %s" % get_filename()
        return

    fiter = iter(f)
    cwd = fiter.next().strip()
    tbtxt = list(fiter)

    # Keep the last line that matches...
    mo = None
    for line in reversed(tbtxt):
        mo = re.match('.*File "(.*)", line ([0-9]+)', line) or mo
        if mo:
            break
    else: # Not found
        return

    if mo:
        filename, lineno = mo.group(1, 2)
        lineno = int(lineno)
        if not isabs(filename):
            filename = realpath(join(cwd, filename))

    print filename, lineno
    invoke_emacs(filename, lineno)
    return 0

def invoke_emacs(filename, lineno):
    subprocess.call(['emacsclient', '-n',
                     '-e', '(find-file "%s")' % filename,
                     '-e', '(goto-line %d)' % lineno,
                     '-e', '(recenter)',
                     '-e', '(set-mark (point))',
                     '-e', '(next-line)'])


if __name__ == '__main__':
    if 'example' in sys.argv:

        # register handler
        import atexit
        atexit.register(atexit_handler)

        def example():
            import time
            print 'count down to error.'
            for i in xrange(10, 1, -1):
                print i
                time.sleep(0.1)
            1/0

        example()
    else:
        sys.exit(invoke_editor())



#!/usr/bin/env python

import os
import sys
import pstats
import cProfile
import tempfile

# TODO:
#  - maybe we should delete the `out` tempfile
#  - I don't like that this currently makes a system call to open the image...
#  - gprof2dot is on the pythonpath we should be able to avoid the system call
def profile_viz(cmd, global_dict=None, local_dict=None, img='profile.png', out='profile.tmp', noctx=False):
    "Run gprof2dot on the output for profiling."
    if noctx:
        cProfile.run(cmd, out)
    else:
        if local_dict is None and global_dict is None:
            call_frame = sys._getframe().f_back
            local_dict = call_frame.f_locals
            global_dict = call_frame.f_globals
        cProfile.runctx(cmd, global_dict, local_dict, out)

    stats = pstats.Stats(out)
    stats.strip_dirs()               # Clean up filenames for the report
    stats.sort_stats('cumulative')   # sort by the cumulative time
    stats.print_stats()
    # for more on the viz check out: http://code.google.com/p/jrfonseca/wiki/Gprof2Dot
    os.system('gprof2dot.py -f pstats %s | dot -Tpng -o %s && eog %s &' % (out, img, img))


def kcachegrind(cmd, out='profile.kgrind'):
    from arsenal.profiling.lsprofcalltree import KCacheGrind
    p = cProfile.Profile()
    p.run(cmd)
    # Get the stats in a form kcachegrind can use and save it
    k = KCacheGrind(p)
    with file(out, 'wb') as f:
        k.output(f)
    os.system("kcachegrind %s &" % out)


def main():
    from optparse import OptionParser
    parser = OptionParser()
    parser.allow_interspersed_args = False
    parser.add_option('-o', '--outfile', dest="outfile",
                      help="Save stats to <outfile>",
                      default='/tmp/profile.tmp')

    if not sys.argv[1:]:
        parser.print_usage()
        sys.exit(2)

    (options, args) = parser.parse_args()

    viz = kcachegrind
    #viz = profile_viz

    sys.path = [os.getcwd()] + sys.path

    if len(args) > 0:
        sys.argv[:] = args
        sys.path.insert(0, os.path.dirname(sys.argv[0]))
        viz('execfile(%r)' % sys.argv[0], out=options.outfile)
    else:
        parser.print_usage()
    return parser


if __name__ == '__main__':
    main()



try:
    from fabulous import color
except ImportError:
    pass

# TODO: Consider using fabulous for most of this. It supports a wider variety of
# colors and might do a better job with other macs.

normal = '\033[0m%s\033[0m'
bold = '\033[1m%s\033[0m'

black, red, green, yellow, blue, magenta, cyan, white = \
    map('\033[3%sm%%s\033[0m'.__mod__, range(8))

light_black, light_red, light_green, light_yellow, light_blue, light_magenta, light_cyan, light_white = \
    map('\033[1;3%sm%%s\033[0m'.__mod__, range(8))

bg_black, bg_red, bg_green, bg_yellow, bg_blue, bg_magenta, bg_cyan, bg_white = \
    map('\033[4%sm%%s\033[0m'.__mod__, range(8))

def padr(w):
    "get format to pad right elements"
    return '%%%ss' % w

pad = padr

def padl(w):
    "get format to pad left elements"
    return '%%-%ss' % w

def getwidth(a):
    "Find maximum width of the string representations of the elements of ``a``."
    return max(len(str(z)) for z in a)


underline = '\033[4m%s\033[0m'

def test():
    for c in 'black, red, green, yellow, blue, magenta, cyan, white'.split(', '):
        print globals()[c] % c
        print globals()['light_' + c] % ('light_' + c)

    for c in 'black, red, green, yellow, blue, magenta, cyan, white'.split(', '):
        print globals()['bg_' + c] % c

    print underline % 'underline'


# TODO: needs some work, but it's pretty fun to use
def color01(x, fmt='%.10f'):
    "Colorize numbers in [0,1] based on value; darker means smaller value."
    if not (0 <= x <= 1 + 1e-10):
        return red % fmt % x
    a, b = 238, 255   # 232, 255
    w = b - a
    offset = x*w
    offset = int(round(offset))
    return color.fg256(a + offset, fmt % x)

if __name__ == '__main__':
    test()



#!/usr/bin/python
"""
Bloom filters in Python, using SHA-1 and Python longs.

My first attempt stored the whole filter in a single arbitrary-size integer,
but for some reason that was 100x slower than storing it in a bunch of 256-bit
integers.
"""

from hashlib import sha1

def nbits_required(n):
    """ Calculate the number of bits required to represent any integer in [0, n). """
    n -= 1
    rv = 0
    while n:
        n >>= 1
        rv += 1
    return rv

class Bloom:
    """ Bloom filter: compact hash table for membership tests with false pos. """

    def __init__(self, size, nhashes, bucketbits=256):
        """
        size: number of bits.  Should be a power of 2.
        nhashes: number of separate hashes to use.

        Making nhashes larger will make it slower.  There are also tradeoffs
        between size, performance, and false-positive rate, which you can look
        up elsewhere.

        default bits per bucket is 256 to cut down on pickle overhead
        """
        self.bucketbits = bucketbits
        self.filter = [0L] * int((size + bucketbits - 1) / bucketbits)
        self.size = size
        self.nhashes = nhashes
        self.hashbits = nbits_required(size)
        assert self.hashbits * nhashes <= 160  # 160's all we get with SHA1

    def add(self, astr):
        """ Add a string to the membership of the filter. """
        for offset in self._hashes(astr):
            bucket, bit = divmod(offset, self.bucketbits)
            self.filter[bucket] |= (1L << bit)

    def __contains__(self, astr):
        """ Returns true if the string is in the filter or it feels like it. """
        for offset in self._hashes(astr):
            bucket, bit = divmod(offset, self.bucketbits)
            if not self.filter[bucket] & (1L << bit): return 0
        return 1

    def _hashes(self, astr):
        """ The hashes of a particular string. """
        digest = sha1(astr).digest()
        # is there no better way to convert a byte string into a long?!
        hashlong = 0L
        for ch in digest:
            hashlong = (hashlong << 8) | ord(ch)
        rv = []
        mask = (1L << self.hashbits) - 1
        for ii in range(self.nhashes):
            # Note that this will give substantially nonuniform results if
            # self.size is not a power of 2, in order to avoid wasting hash
            # bits or doing long division:
            rv.append((hashlong & mask) % self.size)
            hashlong >>= self.hashbits
        return rv

def test_bloom():
    """ Very basic sanity test for Bloom filter implementation. """
    print 'runnning test_bloom...'

    def ok(a, b): assert a == b, (a, b)
    ok(map(nbits_required, range(1, 10)), [0, 1, 2, 2, 3, 3, 3, 3, 4])
    ok(nbits_required(131072), 17)
    ok(nbits_required(131073), 18)

    b = Bloom(1024, 5)
    assert 'asdf' not in b
    assert 'fdsa' not in b
    b.add('asdf')
    assert 'asdf' in b
    assert 'fdsa' not in b

    # false positives (depends on hash function):
    x = Bloom(8, 3)
    x.add('asdf') # about a 5% chance of false positives
    assert 'asdf' in x
    assert 'fdsa' not in x
    ok(filter(x.__contains__, ['foo%d' % ii for ii in range(25)]), ['foo22'])

    print 'pass.'


def misspellings(passage, WORDS):

    import re, cPickle, sys
    try:
        bf = cPickle.load(file('dict.pkl', 'rb'))
    except IOError:
        # /usr/share/dict/words has 234936 words on this Mac and is 2.4 megs
        print "reading dictionary..."
        words = file(WORDS)
        # 2^21 bits, 8.9 per word, would give us 1.5% false positives with 5
        # hashes or 1.7% with 6, so we use 4194304 = 2^22 bits, or 17.8 per
        # word, for 0.09% false positives; that's still only half a mebibyte,
        # although pickle overhead pushes it up to 559K, 22% of the dictionary.
        bf = Bloom(4194304, 5)  
        for line in words: 
            #bf.add(line[:-1].lower())
            bf.add(line.strip().lower().split('/')[0])

        print 'done reading dictionary'
        try:
            cPickle.dump(bf, file('dict.pkl', 'wb'), 2)
        except:
            pass

    def candidates(word):
        """ Words you might find in the dictionary in English. """
        yield word
        for suffix in ['s', 'ing', 'ed', 'es', 'er', 's', 'ly']:
            if word.endswith(suffix):
                yield word[:-len(suffix)]
        for suffix, repl in [('ed', 'e'), ('er', 'e'), ('ing', 'e'), ('ies', 'y'), ('ied', 'y')]:
            if word.endswith(suffix):
                yield word[:-len(suffix)] + repl

    for word in passage:
        # we drop the "'" because our dictionary has "didnt" but not "didn't"
        for chance in candidates(word.replace("'", '').lower()):
            if chance in bf: break
        else:
            print 'typo: %r' % word
            bf.add(word)
    sys.stdout.write('\n')

if __name__ == '__main__':
    test_bloom()

    import sys
    if sys.platform.startswith('win') or sys.platform == 'nt':
        dic = 'C:/Program Files/Mozilla Firefox/dictionaries/en-US.dic'
    else:
        dic = '/usr/share/dict/words'

    sentence = 'You might have a typo in yur codez'
    print 'sentence:', sentence
    misspellings(sentence.split(), dic)




#!/usr/bin/python
import os, os.path, UserDict
from sqlite3 import dbapi2 as sqlite

class dbdict(UserDict.DictMixin):
    ''' dbdict, a persistent dictionnary-like object for large datasets '''

    def __init__(self,dictName):
        self.db_filename = "dbdict_%s.sqlite" % dictName
        if not os.path.isfile(self.db_filename):
            self.con = sqlite.connect(self.db_filename)
            self.con.execute("create table data (key PRIMARY KEY,value)")
        else:
            self.con = sqlite.connect(self.db_filename)

    def __getitem__(self, key):
        row = self.con.execute("select value from data where key=?",(key,)).fetchone()
        if not row: raise KeyError
        return row[0]

    def __setitem__(self, key, item):
        if self.con.execute("select key from data where key=?",(key,)).fetchone():
            self.con.execute("update data set value=? where key=?",(item,key))
        else:
            self.con.execute("insert into data (key,value) values (?,?)",(key, item))
        self.con.commit()

    def __delitem__(self, key):
        if self.con.execute("select key from data where key=?",(key,)).fetchone():
            self.con.execute("delete from data where key=?",(key,))
            self.con.commit()
        else:
             raise KeyError

    def keys(self):
        return [row[0] for row in self.con.execute("select key from data").fetchall()]


if __name__ == '__main__':

    d = dbdict("mydummydict")
    d["foo"] = "bar"

    # At this point, foo and bar are *written* to disk.
    d["John"] = "doh!"
    d["pi"] = 3.999
    d["pi"] = 3.14159

    # You can access your dictionnary later on:
    d = dbdict("mydummydict")
    del d["foo"]

    if "John" in d:
        print "John is in there !"
    print d.items()



import os
import smtplib
import mimetypes
import email
#from email import MIMEMultipart
#from email import MIMEBase, MIMEText
#from email import MIMEAudio
#from email import MIMEImage
#from email import encode_base64

from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.utils import formatdate


from getpass import getpass

def send_gmail(gmail_user=None, passwd=None, recipient=None, subject='', body='', attachmentFilePaths=tuple()):

    if not gmail_user or not recipient:
        raise AssertionError('Sorry, send_gmail is missing information.')

    passwd = passwd or getpass()

    msg = MIMEMultipart()
    msg['From'] = gmail_user
    msg['To'] = recipient
    msg['Subject'] = subject
    msg.attach(MIMEText(body))

    for attachmentFilePath in attachmentFilePaths:
        msg.attach(get_attachment(attachmentFilePath))

    s = smtplib.SMTP('smtp.gmail.com', 587)
    s.ehlo()
    s.starttls()
    s.ehlo()
    s.login(gmail_user, passwd)
    s.sendmail(gmail_user, recipient, msg.as_string())
    s.close()

    print 'Sent email to %s' % recipient


def get_attachment(attachmentFilePath):
    contentType, encoding = mimetypes.guess_type(attachmentFilePath)

    if contentType is None or encoding is not None:
        contentType = 'application/octet-stream'

    mainType, subType = contentType.split('/', 1)

    with file(attachmentFilePath, 'rb') as f:
        if mainType == 'text':
            attachment = MIMEText(f.read())
        elif mainType == 'message':
            attachment = email.message_from_file(f)
        elif mainType == 'image':
            attachment = MIMEImage(f.read(),_subType=subType)
        elif mainType == 'audio':
            attachment = MIMEAudio(f.read(),_subType=subType)
        else:
            attachment = MIMEBase(mainType, subType)
        attachment.set_payload(f.read())
        #encode_base64(attachment)

    attachment.add_header('Content-Disposition', 'attachment', filename=os.path.basename(attachmentFilePath))
    return attachment


if __name__ == '__main__':
    send_gmail(gmail_user='timsfanmail',
               recipient='tim.f.vieira@gmail.com',
               subject='test subject',
               body='test body.')



import sys
from inspect import isfunction, ismethod, getargs, getargspec, formatargspec
from optparse import OptionParser   # TODO: use argparse
from pprint import pprint
from collections import defaultdict
from debug import ip

def usage(obj):
    name = obj.__name__.split('.')[-1]
    signature = call_signature(obj) or '%s(???)' % name
    doc = '    ' + getdoc(obj)   # todo: indent
    return '\n'.join([signature, doc])


def getdoc(obj, indent=0):
    return (getattr(obj, '__doc__', '') or '').strip()


def call_signature(obj, oname=''):
    """
    Return the definition header for any callable object.

    If any exception is generated, None is returned instead and the
    exception is suppressed.
    """
    oname = oname or obj.__name__
    if isfunction(obj):
        func_obj = obj
    elif ismethod(obj):
        func_obj = obj.im_func
    else:
        # obj is not a Python function'
        return None
    args, varargs, varkw = getargs(func_obj.func_code)
    return oname + formatargspec(args, varargs, varkw, func_obj.func_defaults)


def automain(argv=None, verbose=False, breakin=False, ultraTB=False, pdb=False,
             timemsg=False, available=None, mod=None, main_only=False):
    """
    Automatically create a very simple command-line interface.

    Note: All functions must take string arguments
    """

    if not mod:
        import __main__ as mod
    argv = argv or sys.argv

    try:
        if breakin:
            from debug import breakin
            breakin.enable()

        if ultraTB:
            from debug import ultraTB2
            ultraTB2.enable(include_vars=False)

        if pdb:
            from debug.utils import enable_debug_hook
            enable_debug_hook()

        if timemsg:
            from humanreadable import print_elapsed_time
            print_elapsed_time()

    except ImportError:
        pass


    if available:
        filterfn = None
    else:
        def filterfn(obj):
            try:
                if not obj.__module__ == '__main__' and main_only:
                    return False
                if not hasattr(obj, '__call__'):
                    return False
            except AttributeError:
                return False
            else:
                name = obj.__name__
                return name not in ('automain', 'usage', 'call_signature') and not name.startswith('_')
        available = dir(mod)
    objects = [getattr(mod, x) if isinstance(x, basestring) else x for x in available]
    objects = list(sorted(filter(filterfn, objects), key=lambda x: x.__name__))


    # should we print the module's docstring in the help?
    def show_help(objects):
        print
        for obj in objects:
            print usage(obj)
            print

    try:
        action = getattr(mod, argv[1])
        assert hasattr(action, '__call__')
    except (IndexError, AttributeError, AssertionError):
        show_help(objects)
    else:
        args = list(argv[2:])

        parser = OptionParser()
        #parser.allow_interspersed_args = False
        spec = getargspec(action)

        if spec.defaults:
            for default, arg in zip(reversed(spec.defaults), reversed(spec.args)):  # args with default values
                longname = '--' + arg
                #if isinstance(default, bool):   # XXX: this not the right semantics.
                #    if default == False:
                #        parser.add_option(longname, action="store_true", default=False)
                #    else:
                #        parser.add_option(longname, action="store_false", default=True)
                #else:
                parser.add_option(longname, default=default)

        parser.usage = '%s [options] %s' % (action.__name__, ' '.join(spec.args)) \
            + '\n\n    ' + getdoc(action)

        (kw, args) = parser.parse_args(args)

        # TODO: need a better usage message which shows positional arguments

        # minimum non-keyword args
        if len(args) < len(spec.args) - len(spec.defaults or []):
            parser.print_help()
            return

        kw = kw.__dict__

        # call function
        out = action(*args, **kw)

        # pretty-print result
        if isinstance(out, basestring):
            print out

        elif isinstance(out, dict):
            if isinstance(out, defaultdict):
                out = dict(out)
            pprint(out)

        else:
            try:
                out = iter(out)
            except TypeError:
                if out is not None:
                    print out
            else:
                for x in out:
                    print x


if __name__ == '__main__':
    if len(sys.argv) > 1 and sys.argv[1].endswith('.py'):
        sys.argv = sys.argv[1:]
        execfile(sys.argv[0])
        automain()




def main():

    import smtplib
    from email.mime.text import MIMEText
    from email.mime.multipart import MIMEMultipart
    from email.utils import formatdate

    msg = MIMEMultipart()
    msg['From']    = 'tim.f.vieira@gmail.com'
    msg['To']      = 'tim.f.vieira@gmail.com'
    msg['Date']    = formatdate(localtime=True)
    msg['Subject'] = 'Experiment'
    msg.attach(MIMEText("""

    Do *not* drink the coffee -- it's POISONED.

    """))

    s = smtplib.SMTP('loki.cs.umass.edu')
    s.set_debuglevel(1)

    print 'sending...'
    s.sendmail(msg['From'], msg['To'], msg.as_string())
    s.quit()
    print 'done.'


if __name__ == '__main__':
    main()



import os, sys

try:
    import readline
except ImportError:
    pass

def substring_completer(values, normalize=lambda x: x.lower()):
    completions = {}
    values = [(v, normalize(v)) for v in values]

    def completer(text, state):
        try:
            matches = completions[text]   # timv: caching (maybe we don't want to do this.. maybe use an LRU)
        except KeyError:
            matches = [v for v, normalized_v in values if normalize(text) in normalized_v]
            completions[text] = matches
        try:
            return matches[state]
        except IndexError:
            return None

    readline.set_completer(completer)
    readline.parse_and_bind('tab: menu-complete')


def console_width(minimum=None, default=80):
    """Return width of available window area. Autodetection works for
    Windows and POSIX platforms. Returns 80 for others

    Code based on http://bitbucket.org/techtonik/python-pager
    """
    if os.name == 'nt':
        STD_INPUT_HANDLE  = -10
        STD_OUTPUT_HANDLE = -11
        STD_ERROR_HANDLE  = -12

        # get console handle
        from ctypes import windll, Structure, byref
        from ctypes.wintypes import SHORT, WORD, DWORD
        console_handle = windll.kernel32.GetStdHandle(STD_OUTPUT_HANDLE)

        # CONSOLE_SCREEN_BUFFER_INFO Structure
        class COORD(Structure):
            _fields_ = [("X", SHORT), ("Y", SHORT)]

        class SMALL_RECT(Structure):
            _fields_ = [("Left", SHORT), ("Top", SHORT),
                        ("Right", SHORT), ("Bottom", SHORT)]

        class CONSOLE_SCREEN_BUFFER_INFO(Structure):
            _fields_ = [("dwSize", COORD),
                        ("dwCursorPosition", COORD),
                        ("wAttributes", WORD),
                        ("srWindow", SMALL_RECT),
                        ("dwMaximumWindowSize", DWORD)]

        sbi = CONSOLE_SCREEN_BUFFER_INFO()
        ret = windll.kernel32.GetConsoleScreenBufferInfo(console_handle, byref(sbi))
        if ret == 0:
            return max(minimum, 0)
        return max(minimum, sbi.srWindow.Right+1)

    elif os.name == 'posix':
        from fcntl import ioctl
        from termios import TIOCGWINSZ
        from array import array

        winsize = array("H", [0] * 4)
        try:
            ioctl(sys.stdout.fileno(), TIOCGWINSZ, winsize)
        except IOError:
            pass
        return max(minimum, (winsize[1], winsize[0])[0])

    # the fall back value
    return max(minimum, default)


def marquee(msg=''):
    return ('{0:*^%s}' % console_width()).format(msg)


if __name__ == '__main__':

    w = console_width()
    print 'Console width:', w
    print '='*w

    def example():

        values = ['Paul Eden <paul@domain.com>',
                  'Eden Jones <ejones@domain.com>',
                  'Somebody Else <somebody@domain.com>']
        print 'substring completer example:'
        for x in values:
            print x

        substring_completer(values)

        while 1:
            a = raw_input('> ')
            print 'said:', a

    example()





class Alphabet(object):
    """
    Bijective mapping from strings to integers.

    >>> a = Alphabet()
    >>> [a[x] for x in 'abcd']
    [0, 1, 2, 3]
    >>> map(a.lookup, range(4))
    ['a', 'b', 'c', 'd']

    >>> a.stop_growth()
    >>> a['e']

    >>> a.freeze()
    >>> a.add('z')
    Traceback (most recent call last):
      ...
    ValueError: Alphabet is frozen. Key "z" not found.

    >>> print a.plaintext()
    a
    b
    c
    d
    """

    def __init__(self):
        self._mapping = {}   # str -> int
        self._flip = {}      # int -> str; timv: consider using array or list
        self._i = 0
        self._frozen = False
        self._growing = True

    def __repr__(self):
        return 'Alphabet(size=%s,frozen=%s)' % (len(self), self._frozen)

    def freeze(self):
        self._frozen = True

    def stop_growth(self):
        self._growing = False

    @classmethod
    def from_iterable(cls, s):
        inst = cls()
        for x in s:
            inst.add(x)
        inst.freeze()
        return inst

    def keys(self):
        return self._mapping.iterkeys()

    def imap(self, seq, emit_none=False):
        """
        Apply alphabet to sequence while filtering. By default, `None` is not
        emitted, so the Note that the output sequence may have fewer items.
        """
        if emit_none:
            for s in seq:
                yield self[s]
        else:
            for s in seq:
                x = self[s]
                if x is not None:
                    yield x

    def map(self, seq, *args, **kwargs):
        return list(self.imap(seq, *args, **kwargs))

    def add_many(self, x):
        for k in x:
            self.add(k)

    def lookup(self, i):
        if i is None:
            return None
#        assert isinstance(i, int)
        return self._flip[i]

    def lookup_many(self, x):
        for k in x:
            yield self.lookup(k)

    def __contains__(self, k):
        assert isinstance(k, basestring)
        return k in self._mapping

    def __getitem__(self, k):
        try:
            return self._mapping[k]
        except KeyError:
            if not isinstance(k, basestring):
                raise ValueError("Invalid key (%s): only strings allowed." % (k,))
            if self._frozen:
                raise ValueError('Alphabet is frozen. Key "%s" not found.' % (k,))
            if not self._growing:
                return None
            x = self._mapping[k] = self._i
            self._flip[x] = k
            self._i += 1
            return x

    add = __getitem__

    def __iter__(self):
        for i in xrange(len(self)):
            yield self._flip[i]

    def enum(self):
        for i in xrange(len(self)):
            yield (i, self._flip[i])

    def __len__(self):
        return len(self._mapping)

    def plaintext(self):
        return '\n'.join(self)

    @classmethod
    def load(cls, filename):
        with file(filename) as f:
            return cls.from_iterable(l.strip() for l in f)

    def save(self, filename):
        with file(filename, 'wb') as f:
            f.write(self.plaintext())


if __name__ == '__main__':
    import doctest
    doctest.testmod()



#!/usr/bin/env python

import re
import sys
from urllib2 import urlopen

def readurl(url):
    return urlopen(url).read()

## def oanda(fromcurr, tocurr):
##     # http://www.oanda.com/currency/historical-rates?expr2=EUR&exch2=USD&format=ASCII&date1=01/13/11&date=01/19/11&lang=en&result=1
##     fromdate = '01/13/11'
##     todate = '01/19/11'
##     url = 'http://www.oanda.com/currency/historical-rates?lang=en&result=1&format=ASCII'
##     url += '&exch2=%s&expr2=%s' % (fromcurr, tocurr)
##     url += '&date1=%s&date=%s' % (fromdate, todate)
##     content = readurll(url)
##     pre = re.findall('<pre.*>([\w\W]*?)</pre>', content, re.IGNORECASE)
##     assert len(pre) == 1
##     print [(a, float(b)) for (a,b) in (x.split() for x in pre[0].strip().split('\n'))]

def google(currencyFrom, currencyTo):
    assert currencyFrom != currencyTo
    url = 'http://www.google.com/ig/calculator?hl=en&q=1' + currencyFrom + '%3D%3F' + currencyTo
    content = readurl(url)
    try:
        [result] = re.findall(".*rhs:\s*\"(\d\.\d*)", content)
        print currencyFrom + " converted to " + currencyTo + ": %s" % float(result)
    except ValueError:
        print 'ERROR:', content

if __name__ == '__main__':
    try:
        f, t = sys.argv[1:]
    except ValueError:
        print 'usage: %s <from> <to>' % sys.argv[0]
        sys.exit(1)
    google(f,t)



#!/usr/bin/env python
#
#  Script for getting Google Page Rank of page
#  Google Toolbar 3.0.x/4.0.x Pagerank Checksum Algorithm
#
#  original from http://pagerank.gamesaga.net/
#  this version was adapted from http://www.djangosnippets.org/snippets/221/
#  by Corey Goldberg - 2010
#
#  Licensed under the MIT license: http://www.opensource.org/licenses/mit-license.php




from urllib import quote

from arsenal.web.download import urlread


def get_pagerank(url):
    hsh = check_hash(hash_url(url))
    gurl = 'http://www.google.com/search?client=navclient-auto&features=Rank:&q=info:%s&ch=%s' % (quote(url), hsh)
    _, _, contents = urlread(gurl)
    rank = contents.strip()[9:]
    return rank


def  int_str(string, integer, factor):
    for i in range(len(string)) :
        integer *= factor
        integer &= 0xFFFFFFFF
        integer += ord(string[i])
    return integer


def hash_url(string):
    c1 = int_str(string, 0x1505, 0x21)
    c2 = int_str(string, 0, 0x1003F)

    c1 >>= 2
    c1 = ((c1 >> 4) & 0x3FFFFC0) | (c1 & 0x3F)
    c1 = ((c1 >> 4) & 0x3FFC00) | (c1 & 0x3FF)
    c1 = ((c1 >> 4) & 0x3C000) | (c1 & 0x3FFF)

    t1 = (c1 & 0x3C0) << 4
    t1 |= c1 & 0x3C
    t1 = (t1 << 2) | (c2 & 0xF0F)

    t2 = (c1 & 0xFFFFC000) << 4
    t2 |= c1 & 0x3C00
    t2 = (t2 << 0xA) | (c2 & 0xF0F0000)

    return (t1 | t2)


def check_hash(hash_int):
    hash_str = '%u' % (hash_int)
    flag = 0
    check_byte = 0

    i = len(hash_str) - 1
    while i >= 0:
        byte = int(hash_str[i])
        if 1 == (flag % 2):
            byte *= 2;
            byte = byte / 10 + byte % 10
        check_byte += byte
        flag += 1
        i -= 1

    check_byte %= 10
    if 0 != check_byte:
        check_byte = 10 - check_byte
        if 1 == flag % 2:
            if 1 == check_byte % 2:
                check_byte += 9
            check_byte >>= 1

    return '7' + str(check_byte) + hash_str



if __name__ == '__main__':
    import sys
    if len(sys.argv) == 2:
        print get_pagerank(sys.argv[1])
    else:
        print 'usage: python %s <URL>\n' % sys.argv[0]



#!/usr/bin/env python
import re
import cgi

colorcodes =   {'bold':{True:'\033[1m',False:'\033[22m'},
                'cyan':{True:'\033[36m',False:'\033[39m'},
                'blue':{True:'\033[34m',False:'\033[39m'},
                'red':{True:'\033[31m',False:'\033[39m'},
                'magenta':{True:'\033[35m',False:'\033[39m'},
                'green':{True:'\033[32m',False:'\033[39m'},
                'underline':{True:'\033[4m',False:'\033[24m'}}

def recolor(color, text):
#    regexp = "(?:%s)(.*?)(?:%s)" % (colorcodes[color][True], colorcodes[color][False])
    regexp = "(?:%s)(.*?)(?:%s)" % (colorcodes[color][True], '\033[0m')
    regexp = regexp.replace('[', r'\[')
    return re.sub(regexp, r'''<span style="color: %s">\1</span>''' % color, text)

def bold(text):
    regexp = "(?:%s)(.*?)(?:%s)" % (colorcodes['bold'][True], '\033[0m')
    regexp = regexp.replace('[', r'\[')
    return re.sub(regexp, r'<span style="font-weight:bold">\1</span>', text)

def underline(text):
    regexp = "(?:%s)(.*?)(?:%s)" % (colorcodes['underline'][True], '\033[0m')
    regexp = regexp.replace('[', r'\[')
    return re.sub(regexp, r'<span style="text-decoration: underline">\1</span>', text)

def removebells(text):
    return text.replace('\07', '')

def removebackspaces(text):
    backspace_or_eol = r'(.\010)|(\033\[K)'
    n = 1
    while n > 0:
        text, n = re.subn(backspace_or_eol, '', text, 1)
    return text

template = '''\
<html>
<head>
    <style text="text/css">
    body {
        font-family:    Monaco,
                        "Bitstream Vera Sans Mono",
                        "Lucida Console",
                        Terminal,
                        monospace;
        font-size:      14;
    }
    </style>
</head>
<body>
%s
</body>
</html>
'''

re_string = re.compile(r'(?P<htmlchars>[<&>])|(?P<space>^[ \t]+)|(?P<lineend>\r\n|\r|\n)|(?P<protocal>(^|\s)((http|ftp)://.*?))(\s|$)', re.S|re.M|re.I)
def ansi2html(text, tabstop=4, template=template):
    def do_sub(m):
        c = m.groupdict()
        if c['htmlchars']:
            return cgi.escape(c['htmlchars'])
        if c['lineend']:
            return '<br>'
        elif c['space']:
            t = m.group().replace('\t', '&nbsp;'*tabstop)
            t = t.replace(' ', '&nbsp;')
            return t
        elif c['space'] == '\t':
            return ' '*tabstop;
        else:
            url = m.group('protocal')
            if url.startswith(' '):
                prefix = ' '
                url = url[1:]
            else:
                prefix = ''
            last = m.groups()[-1]
            if last in ['\n', '\r', '\r\n']:
                last = '<br>'
            return '%s%s' % (prefix, url)
    result = re.sub(re_string, do_sub, text)
    result = recolor('cyan', result)
    result = recolor('blue', result)
    result = recolor('red', result)
    result = recolor('magenta', result)
    result = recolor('green', result)
    result = bold(result)
    result = underline(result)
    result = removebells(result)
    result = removebackspaces(result)

    return template % result


if __name__ == '__main__':
    import sys
    if len(sys.argv) == 2:
        with open(sys.argv[1]) as f:
            print ansi2html(f.read())
    else:
        with sys.stdin as f:
            print ansi2html(f.read())




"""
wchoice.py -- by bearophile, V.1.0 Oct 30 2006

Weighted choice: like the random.choice() when the probabilities of
the single elements aren't the same.
"""

from random import random
from bisect import bisect
from itertools import izip

from numpy import searchsorted, cumsum, array
from numpy.random import random


## TODO: maybe we should merge these two functions into something awesome.
def faster_weighted_choice(p, n):
    """
    Generate n samples of the indices of the vector of probabilites p
    sampled *indicies* are placed in an array of length n.
    """
    # uniformly distributed random vector of length n
    y = random((n,))
    # cumulative probability vector
    c = cumsum(p)
    return searchsorted(c, y)

if __name__ == '__main__':
    p = array((0.1, 0.2, 0.6, 0.1))  # vector of probabilities, normalized to 1
    print faster_weighted_choice(p, 10)     # generate 10 samples



def wchoice(objects, frequences, filter=True, normalize=True, return_index=False):
    """
    wchoice(objects, frequences, filter=True, normalize=True): return
    a function that return the given objects with the specified frequency
    distribution. If no objects with frequency>0 are given, return a
    constant function that return None.

    Input:
      objects: sequence of elements to choose.
      frequences: sequence of their frequences.
      filter=False disables the filtering, speeding up the object creation,
        but less bad cases are controlled. Frequences must be float > 0.
      normalize=False disables the probablitity normalization. The choice
        becomes faster, but sum(frequences) must be 1
    """
    if filter:
        # Test and clean the frequencies.
        if isinstance(frequences, (set, dict)):
            raise "in wchoice: frequences: only ordered sequences."
        if isinstance(objects, (set, dict)):
            raise "in wchoice: objects: only ordered sequences."
        if len(frequences) != len(objects):
            raise "in wchoice: objects and frequences must have the same lenght."
        frequences = map(float, frequences)
        filteredFreq = []
        filteredObj = []
        for freq, obj in izip(frequences, objects):
            if freq < 0:
                raise "in wchoice: only positive frequences."
            elif freq >1e-8:
                filteredFreq.append(freq)
                filteredObj.append(obj)

        if len(filteredFreq) == 0:
            return lambda: None
        if len(filteredFreq) == 1:
            return lambda: filteredObj[0]
        frequences = filteredFreq
        objects = filteredObj
    else:
        if len(objects) == 1:
            return lambda: objects[0]
        # Here objects is unaltered, so it must have a fast __getitem__

    addedFreq = []
    lastSum = 0
    for freq in frequences:
        lastSum += freq
        addedFreq.append(lastSum)

    if return_index:
        return lambda : bisect(addedFreq, random()*lastSum)

    # If the choice method is called many times, then the frequences
    # can be normalized to sum 1, so instead of random()*self.sumFreq
    # a random() suffices.
    if normalize:
        return lambda rnd=random, bis=bisect: objects[bis(addedFreq, rnd()*lastSum)]
    else:
        return lambda rnd=random, bis=bisect: objects[bis(addedFreq, rnd())]


if __name__ == '__main__':
    print "wchoice tests:"
    objs = "ABCDE"
    freqs = [1, 3, 1.1, 0, 5]
    sumf = sum(freqs)
    wc = wchoice(objs, freqs)
    freq1 = dict.fromkeys(objs, 0)
    nestractions = 100000
    for i in xrange(nestractions):
        freq1[wc()] += 1

    freq2 = sorted(freq1.items())
    freq3 = [sumf*float(v)/nestractions for (k,v) in freq2]

    for (f1,f2) in zip(freq3, freqs):
        print abs(f1-f2),
        assert abs(f1-f2) < 0.05
    print "\n"

    wc = wchoice(["a"], [1])
    assert set(wc() for i in xrange(20000)) == set(["a"])

    wc = wchoice(["a"], [0])
    assert set(wc() for i in xrange(20000)) == set([None])

    wc = wchoice(["a","b"], [0,0])
    assert set(wc() for i in xrange(20000)) == set([None])

    objs = ["A"]
    freqs = [1.5]
    wc = wchoice(objs, freqs, filter=False)
    assert [wc() for _ in xrange(10)] == ["A"] * 10

    objs = "ABCDE"
    freqs = [1, 3, 1.1, 0.1, 5]
    wc = wchoice(objs, freqs, filter=False)
    print [wc() for _ in xrange(50)]

    print "Tests done."



"""
Draw inheritance hierarchies via Dot (http://www.graphviz.org/)
Author: Michele Simionato
E-mail: mis6@pitt.edu
Date: August 2003
License: Python-like
Requires: Python 2.3, dot, standard Unix tools
"""

import os
import itertools

PSVIEWER = 'evince'
PNGVIEWER = 'kview'
PSFONT = 'Times'
PNGFONT = 'Courier'


def MRO(cls):
    "Returns the MRO of cls as a text"
    out = ["MRO of %s:" % cls.__name__]
    for counter, c in enumerate(cls.__mro__):
        name = c.__name__
        bases = ','.join([b.__name__ for b in c.__bases__])
        s = "  %s - %s(%s)" % (counter, name, bases)
        if type(c) is not type:
            s += "[%s]" % type(c).__name__
        out.append(s)
    return '\n'.join(out)


class MROgraph(object):

    def __init__(self, *classes, **options):
        """Generates the MRO graph of a set of given classes."""
        if not classes:
            raise Exception('Missing class argument!')
        filename = options.get('filename', "MRO_%s.ps" % classes[0].__name__)
        self.labels = options.get('labels', 2)
        caption = options.get('caption', '')
        setup = options.get('setup', '')
        name, dotformat = os.path.splitext(filename)
        format = dotformat[1:]
        fontopt = "fontname=" + (PSFONT if format == 'ps' else PNGFONT)
        nodeopt = ' node [%s];\n' % fontopt
        edgeopt = ' edge [%s];\n' % fontopt
        viewer = PSVIEWER if format == 'ps' else PNGVIEWER
        self.textrepr = '\n'.join([MRO(cls) for cls in classes])
        if caption:
            caption = ('caption [shape=box,label="%s\n",fontsize=9];'
                       % self.textrepr).replace('\n', '\\l')
        setupcode = nodeopt + edgeopt + caption + '\n' + setup + '\n'
        codeiter = itertools.chain(*[self.genMROcode(cls) for cls in classes])
        self.dotcode = \
            'digraph %s{\n%s%s}' % (name, setupcode, '\n'.join(codeiter))
        os.system("echo '%s' | dot -T%s > %s; %s %s&" % (self.dotcode,
                                                         format,
                                                         filename,
                                                         viewer,
                                                         filename))

    def genMROcode(self, cls):
        "Generates the dot code for the MRO of a given class"
        for mroindex, c in enumerate(cls.__mro__):
            name = c.__name__
            manyparents = len(c.__bases__) > 1

            if c.__bases__:
                yield ''.join(' edge [style=solid]; %s -> %s %s;\n' % (
                    b.__name__, name,
                    ('[label="%s"]' % (i+1)) \
                        if manyparents and self.labels == 2 \
                        else '')
                    for i, b in enumerate(c.__bases__))

            if manyparents:
                yield " {rank=same; %s}\n" \
                    % ''.join('"%s"; ' % b.__name__ for b in c.__bases__)

            number = "%s-" % mroindex if self.labels else ''
            label = 'label="%s"' % (number + name)

            # if cls is a metaclass use a box
            option = '[%s]' if issubclass(cls, type) else '[shape=box,%s]'

            yield(' %s %s;\n' % (name, option % label))

            if type(c) is not type: # c has a custom metaclass
                metaname = type(c).__name__
                yield ' edge [style=dashed]; %s -> %s;' % (metaname, name)

    def __repr__(self):
        "Returns the Dot representation of the graph"
        return self.dotcode

    def __str__(self):
        "Returns a text representation of the MRO"
        return self.textrepr



if __name__ == "__main__":

    def example():
        class M(type): pass # metaclass
        class F(object): pass
        class E(object): pass
        class D(object): pass
        class G(object): __metaclass__ = M
        class C(F,D,G): pass
        class B(E,D): pass
        class A(B,C): pass
        print MROgraph(A,M)

    example()



from collections import defaultdict

class LexiconToken(object):

    def __init__(self, w):
        self.form = w
        self.next = None
        self.prev = None

    def __repr__(self):
        return 'LexiconToken(%s)' % self.form


class Lexicon(object):

    def __init__(self, filename, key=lambda x: x.lower()):
        self.contents = defaultdict(list)
        self.key = key
        with file(filename, 'r') as f:
            for line in f:
                line = line.strip()
                if line:
                    # create LexiconTokens and set their next and prev pointers
                    tokens = []
                    t = None
                    for w in line.split():
                        t2 = LexiconToken(self.key(w))
                        t2.prev = t
                        if t is not None:
                            t.next = t2
                        t = t2
                        tokens.append(t2)
                    # add tokens to lexicon
                    for tk in tokens:
                        self.__iadd__(tk)

    def __iadd__(self, t):
        if isinstance(t, basestring):
            t = LexiconToken(self.key(t))
        self.contents[self.key(t.form)].append(t)

    def contains(self, query):
        """Is query in the lexicon, accounting for lexicon phrases and the context."""

        def xx(x):
            fullentry = []
            while x:
                fullentry.append(x)
                x = x.next
            return fullentry

        for entry in self.contents[self.key(query.form)]:

            te = entry
            tq = query
            result = True

            # Go the beginning of this lexicon entry
            while te.prev and result:
                if not tq.prev:
                    return False
                te = te.prev
                tq = tq.prev

            # Check for match all the way to the end of this lexicon entry
            doWhileHack = True
            while te and tq and result or doWhileHack:
                if te.form != self.key(tq.form):
                    result = False
                te = te.next
                tq = tq.next
                doWhileHack = False
            if result and not te:
                return True

        return False


def test():
    lex = Lexicon('lexicon/US-states')

    from textmill.representation import Instance

    seq = Instance.from_sgml("Welcome to South DaKota")
    assert lex.contains(seq.sequence[2])
    assert lex.contains(seq.sequence[3])

    seq = Instance.from_sgml("South")
    assert not lex.contains(seq.sequence[0])

    seq = Instance.from_sgml("North")
    assert not lex.contains(seq.sequence[0])


if __name__ == '__main__':
    test()
